"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs = __importStar(require("yargs"));
const token = __importStar(require("./lib/token"));
const drives = __importStar(require("./lib/drives"));
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path_1 = require("path");
const GRAPH = 'https://graph.microsoft.com/v1.0';
const TMP_FILE = path_1.join(os.tmpdir(), 'odrive.json');
const argv = yargs.usage('Usage $0 <cmd>')
    .command('auth', 'show auth url', (cmd) => {
    return cmd.positional('key', {
        describe: 'client key',
    }).positional('secret', {
        describe: 'client secret',
    }).positional('code', {
        describe: 'code from url after auth'
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    if (argv.code) {
        yield token.exToken(argv.code, argv.key, argv.secret);
    }
    else {
        return token.AuthUrl(argv.key, 'offline_access files.readwrite.all');
    }
}))
    .command('drive', 'show dirives', (cmd) => {
    return cmd.positional('user', {
        alias: 'u',
        default: '/me/drive',
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    const info = yield token.getToken();
    const dv = yield drives.meDrive(GRAPH, info.access_token);
    console.log('drive %j', dv);
}))
    .command('item', 'show items', (cmd) => {
    return cmd.positional('path', {
        alias: 'p',
        default: '/',
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    const info = yield token.getToken();
    const dv = yield drives.itemList(GRAPH, info.access_token, argv.path);
    dv.forEach((e, ii) => {
        const ty = e.file ? e.file.mimeType : (e.folder ? 'folder/' + e.folder.childCount : 'unkonwn');
        let ss = e.size + 'B';
        if (e.size > 1024 * 1024) {
            ss = Math.ceil(e.size / 1024 / 1024) + 'M';
        }
        else if (e.size > 1024) {
            ss = Math.ceil(e.size / 1024) + 'K';
        }
        console.log('%d %s (%s) %s %s', ii, e.id, ss, ty, e.name);
    });
    fs.writeFile(TMP_FILE, JSON.stringify(dv.map(e => e.id)), (err) => err);
}))
    .command('get <id>', 'download file', (cmd) => {
    return cmd.positional('id', {
        type: 'string',
    }).positional('path', {
        alias: 'p',
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    let id = argv.id;
    if (id.startsWith('+')) {
        const ss = JSON.parse(fs.readFileSync(TMP_FILE).toString());
        id = ss[+id];
    }
    console.log('GET', id);
    const info = yield token.getToken();
    const it = yield drives.itemInfo(GRAPH, info.access_token, id);
    if (it.file) {
        let ip = argv.path;
        ip = path_1.join(process.cwd(), ip);
        const st = yield token.fsStat(ip);
        if (st) { // access
            if (st.isDirectory()) {
                ip = path_1.join(ip, it.name);
            }
        }
        console.log(`curl -o ${JSON.stringify(ip)} '${it.webUrl}'`);
    }
}))
    .command('del <id>', 'delete remote file', (cmd) => {
    return cmd.positional('id', {
        type: 'string',
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    let id = argv.id;
    if (id.startsWith('+')) {
        const ss = JSON.parse(fs.readFileSync(TMP_FILE).toString());
        id = ss[+id];
    }
    console.log('DEL', id);
    const info = yield token.getToken();
    yield drives.itemDel(GRAPH, info.access_token, id);
}))
    .command('put <file>', 'upload file', (cmd) => {
    return cmd.positional('file', {
        describe: 'local path',
    }).positional('dir', {
        alias: 'd',
        describe: 'remote dir',
    });
}, (argv) => __awaiter(this, void 0, void 0, function* () {
    const fpath = argv.file;
    const st = yield token.fsStat(fpath);
    if (!st)
        return console.log('not exists', argv.file);
    let rpath = path_1.basename(fpath);
    if (argv.dir) {
        rpath = path_1.join(argv.dir, rpath);
    }
    console.log('PUT', argv.file, 'TO', rpath);
    const info = yield token.getToken();
    const sesson = yield drives.uploadSession(GRAPH, info.access_token, rpath);
    const item = yield drives.uploadIter(sesson.uploadUrl, fs.openSync(fpath, 'r'), st.size);
    if (item && item.file) {
        console.log(item.id, item.file.mimeType, item.name);
    }
}))
    .option('log', {
    default: 'info',
})
    .fail((msg, error) => {
    if (error.isAxiosError) {
        const { response, config } = error;
        return console.error({ config, data: response.data });
    }
    console.error(error);
})
    .help()
    .argv;
console.log('argv %j', argv);
