"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const fs = __importStar(require("fs"));
function meDrive(root, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const headers = {
            Authorization: 'bearer ' + token
        };
        const resp = yield axios_1.default.get(root + '/me/drive', { headers });
        return resp.data;
    });
}
exports.meDrive = meDrive;
function itemList(root, token, path) {
    return __awaiter(this, void 0, void 0, function* () {
        const headers = {
            Authorization: 'bearer ' + token
        };
        if (path.length > 1) {
            path = `:/${path}:/`;
        }
        const resp = yield axios_1.default.get(root + '/me/drive/root' + path + 'children', { headers });
        return resp.data.value;
    });
}
exports.itemList = itemList;
function itemInfo(root, token, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const headers = {
            Authorization: 'bearer ' + token
        };
        id = encodeURIComponent(id);
        const resp = yield axios_1.default.get(root + '/me/drive/items/' + id, { headers });
        if (!resp.data.folder) {
            const cont = yield axios_1.default.get(root + '/me/drive/items/' + id + '/content', { headers, maxRedirects: 0, validateStatus: i => i < 400 });
            resp.data.webUrl = cont.headers['location'];
        }
        return resp.data;
    });
}
exports.itemInfo = itemInfo;
function itemDel(root, token, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const headers = {
            Authorization: 'bearer ' + token
        };
        id = encodeURIComponent(id);
        const resp = yield axios_1.default.delete(root + '/me/drive/items/' + id, { headers });
        console.log(resp.status, resp.statusText);
    });
}
exports.itemDel = itemDel;
function uploadSession(root, token, path) {
    return __awaiter(this, void 0, void 0, function* () {
        const headers = {
            Authorization: 'bearer ' + token
        };
        path = `:/${encodeURIComponent(path)}:`;
        const resp = yield axios_1.default.post(root + '/me/drive/root' + path + '/createUploadSession', {}, { headers });
        return resp.data;
    });
}
exports.uploadSession = uploadSession;
const SIZE_4M = 4 * 1024 * 1024;
function uploadIter(url, fd, total) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = Buffer.alloc(SIZE_4M);
        let nx = 0;
        let sum = 0;
        do {
            nx = fs.readSync(fd, data, 0, SIZE_4M, sum);
            const headers = {
                'Content-Length': nx,
                'Content-Range': `bytes ${sum}-${sum + nx - 1}/${total}`
            };
            sum += nx;
            console.log('UPLOAD_RANGE %j', { nx, sum, headers });
            const end = nx < SIZE_4M;
            if (end) {
                data = data.slice(0, nx);
            }
            const resp = yield axios_1.default.put(url, data, { headers });
            // console.log('UPLOAD_RESP %j', resp.data)
            if (end) {
                return resp.data;
            }
        } while (nx > 0);
    });
}
exports.uploadIter = uploadIter;
